@@ -1,19 +1,19 @@
 #' Tagloss_model returns the daily rate of tag loss.
 #' @title Return the daily rate of tag loss.
-#' @author Marc Girondot
+#' @author Marc Girondot \email{marc.girondot@@gmail.com}
 #' @return Return the daily rate of tag loss if hessian is null or a data.frame with distribution of daily rate of tag loss if hessian is not null.
 #' @param t Time for which values of model must be estimated
 #' @param x A Tagloss fitted model
 #' @param par Parameters
-#' @param hessian hessian matrix of parameters
+#' @param Hessian Hessian matrix of parameters
+#' @param mcmc A mcmc result
 #' @param model The model of parameter to be used, can be 1, 2, L1, L2, R1 or R2
 #' @param model_before Function to be used before estimation of daily tagloss rate
 #' @param model_after Function to be used after estimation of daily tagloss rate
+#' @param method Can be NULL, "delta", "SE", "Hessian", "MCMC", or "PseudoHessianFromMCMC"
 #' @param replicates Number of replicates to estimate se of output
 #' @description This function compute a model of daily tag loss rate for days t 
-#' based on a set of parameters, par.\cr
-#' If hessian is not null, it will estimate standard error of the output using numerical delta method is replicates 
-#' is null or using resampling if replicates is not null.\cr
+#' based on a set of parameters, par or a fitted tag loss model in x.\cr
 #' Parameters are described in \code{\link{Tagloss_fit}}.
 #' @family Model of Tag-loss
 #' @examples
@@ -24,7 +24,7 @@
 #' t <- 1:1000
 #' par <- c(D1=200, D2D1=100, D3D2=200, 
 #'          A=-logit(0.02), B=-logit(0.05), C=-logit(0.07))
-#' y <- Tagloss_model(t, par)
+#' y <- Tagloss_model(t, par, model="1")
 #' plot(x=t, y, type="l")
 #' par <- c(D1_1=200, D2D1_1=100, D3D2_1=200, 
 #'          A_1=-logit(0.02), B_1=-logit(0.05), C_1=-logit(0.07))
@@ -60,30 +60,89 @@
 #' par <- c(a0=-0.1, a1=-10, a2=0.2, a3=60, a4=0.1)
 #' y <- Tagloss_model(t, par)
 #' plot(x=t, y, type="l")
+#' 
+#' # Example with fitted data
+#' data_f_21 <- Tagloss_format(outLR, model="21")
+#' # Without the N20 the computing is much faster
+#' data_f_21_fast <- subset(data_f_21, subset=(is.na(data_f_21$N20)))
+#' par <- c('D1_2' = 49.086835072129126, 
+#'          'D2D1_2' = 1065.0992647723231, 
+#'          'D3D2_2' = 6.15531475922079, 
+#'          'A_2' = 5.2179675647973758, 
+#'          'B_2' = 8.0045560376751386, 
+#'          'C_2' = 8.4082505219581876, 
+#'          'D1_1' = 177.23337287498103, 
+#'          'D2D1_1' = 615.42690323741033, 
+#'          'D3D2_1' = 2829.0806609455867, 
+#'          'A_1' = 28.500118091731551, 
+#'          'B_1' = 10.175426055942701, 
+#'          'C_1' = 6.9616630417169398)
+#' o <- Tagloss_fit(data=data_f_21_fast, fitted.parameters=par)
+#' t <- 1:10
+#' y <- Tagloss_model(t, o$par, model="1")
+#' y <- Tagloss_model(t, x=o, method=NULL, model="1")
+#' y <- Tagloss_model(t, x=o, method="Hessian", model="1", replicates=1000)
 #' }
 #' @export
 
-Tagloss_model <- function(t, par=NULL, hessian=NULL, model_before = NULL, 
-                          model_after = NULL, 
-                          model=NULL, replicates=NULL, x=NULL) {
+Tagloss_model <- function(t=NULL                                                      , 
+                          par=NULL                                                    , 
+                          Hessian=NULL                                                , 
+                          mcmc = NULL                                                 ,
+                          model_before = NULL                                         , 
+                          model_after = NULL                                          , 
+                          model=stop("You must specify which tag loss rate you want."), 
+                          method=NULL                                                 , 
+                          replicates=NULL                                             , 
+                          x=NULL                                                      ) {
+  
+  if (is.null(replicates)) replicates <- 0
+  if (is.null(method) | (replicates == 0)) method <- "null"
+  method <- tolower(method)
+  method <- match.arg(method, choices = c("null", 
+                                          "delta", 
+                                          "se", 
+                                          "hessian", 
+                                          "mcmc", 
+                                          "pseudohessianfrommcmc"))
   
   if (is.null(x) & is.null(par)) {
     stop("Both par and x cannot be null at the same time")
   }
   
+  if (is.null(t)) {
+    if (is.null(x)) {
+      stop("Both t and x cannot be null at the same time")
+    } else {
+      t <- 1:x$days.maximum
+    }
+  }
+  
   if (!is.null(x)) {
     if (is.null(par)) par <- c(x$par, x$fixed.par)
-    if (is.null(hessian)) hessian <- x$hessian
+    if (is.null(Hessian)) Hessian <- x$hessian
     if (is.null(model_before)) model_before <- x$model_before
     if (is.null(model_after)) model_after <- x$model_after
   }
   
+  if (!is.null(model)) {
+    model <- toupper(model)
+    model <- match.arg(model, choices = c("1", "2", "L1", "L2", "R1", "R2"))
+  } else {
+    stop("Model cannot be NULL.")
+  }
   
-  if (!is.null(model)) model <- toupper(model)
+  if (method=="hessian" & is.null(Hessian)) {
+    stop("Hessian method cannot be used with an empty Hessian matrix.")
+  }
+  
+  if ((method=="mcmc" | method=="pseudohessianfrommcmc") & is.null(mcmc)) {
+    stop("MCMC methods cannot be used without MCMC parameter.")
+  }
   
   days.maximum <- max(t)
   
-  if (is.null(hessian)) {
+  if (method=="null") {
     
     if (!is.null(model_before)) eval(parse(text=model_before), envir= environment())
     
@@ -203,14 +262,20 @@ Tagloss_model <- function(t, par=NULL, hessian=NULL, model_before = NULL,
       z <- get(paste0("p", model))
     }
     
-    as.numeric(ifelse(z<0, 1E-9, ifelse(z>1, 1-1E-9, z)))
+    return(as.numeric(ifelse(z<0, 1E-9, ifelse(z>1, 1-1E-9, z))))
     
     
-  } else {
+  }
+  
+  # Si je suis ici c'est que je dois calculer une erreur standard
+  
+  if (method == "delta") {
+    if (!(is.element('nlWaldTest', installed.packages()[,1]))) stop("nlWaldTest package must be installed to use the delata method.")
     
-    # Je prépare les variables communes pour les deux méthodes
+    suppressPackageStartupMessages(requireNamespace("nlWaldTest"))
+    message("Estimation of distribution using delta method")
     
-    VCov <- solve(hessian)
+    VCov <- solve(Hessian)
     par_hess <- par
     par_hess <- par_hess[colnames(VCov)]
     
@@ -224,198 +289,170 @@ Tagloss_model <- function(t, par=NULL, hessian=NULL, model_before = NULL,
     
     if (any(diag(VCov)<0)) stop("Hessian matrix is not correct; probably you are not at maximum likelihood.")
     
-    if (is.element('MultiRNG', installed.packages()[,1]) & (!is.null(replicates))) {
-      # J'ai hessian, alors j'utilise la méthode resampling
-      suppressPackageStartupMessages( requireNamespace("MultiRNG") )
-      message("Estimation of distribution using resampling method")
+    
+    try_g2 <- function(..., Time, model_before=NULL, model_after=NULL, model, pfixed=NULL) {
       
-      dta <- getFromNamespace("draw.d.variate.normal", ns="MultiRNG")(no.row=replicates,
-                                                                      d=nrow(VCov),
-                                                                      mean.vec=par_hess,
-                                                                      cov.mat=VCov)
-      # Je mets la première ligne avec les valeurs MLE
-      dta[1, ] <- par_hess
+      par <- c(...)
       
-      for (ti in t) {  
-        outr <- NULL
-        for (r in 1:replicates) {
-          outr <- c(outr, Tagloss_model(ti, par=c(dta[r, ], par_add), 
-                                        model_before=model_before, 
-                                        hessian = NULL, model = model))
-        }
-        gh <- rbind(gh, data.frame(time=ti, value=outr[1], mean=mean(outr), se=sd(outr), 
-                                   'X2.5'=unname(quantile(outr, probs=0.025)), 
-                                   'X97.5'=unname(quantile(outr, probs=0.975))))
-      }
-      return(gh)
+      # print(d(par))
       
-    } else {
-      # Je suis en méthode delta
-      if (is.element('nlWaldTest', installed.packages()[,1])) {
-        # J'ai hessian, alors j'utilise la méthode resampling
-        suppressPackageStartupMessages(  requireNamespace("nlWaldTest"))
-        message("Estimation of distribution using delta method")
-        
-        try_g2 <- function(..., Time, model_before=NULL, model_after=NULL, model, pfixed=NULL) {
-          
-          par <- c(...)
-          
-          # print(d(par))
-          
-          tgm <- Tagloss_model(Time, par=c(par, pfixed), 
-                               model_before=model_before,
-                               model_after=model_after, 
-                               hessian = NULL, model = model)
-          
-          return(tgm)
-        }
-        
-        
-        nlConfint2 <- function (obj = NULL, texts, level = 0.95, coeff = NULL, Vcov = NULL, 
-                                df2 = NULL, x = NULL, parameters=NULL) 
-        {
-          if (!is.null(obj)) {
-            co = try(coef(obj), silent = T)
-            cond = attr(co, "condition")
-            if (is.null(coeff) && (is.null(cond))) 
-              coeff = co
-            vc = try(vcov(obj), silent = T)
-            cond2 = attr(vc, "condition")
-            if (is.null(Vcov) && (is.null(cond2))) 
-              Vcov = vc
-          }
-          if (is.null(coeff)) {
-            if (is.null(obj)) 
-              mess = "Both  'obj' and 'coeff' are missing"
-            else {
-              clm = class(obj)
-              part1 = "There are no coef() methods for model objects of class \""
-              mess = paste0(part1, clm, "\".\nInput the 'coeff' parameter.")
-            }
-            stop(mess)
-          }
-          if (is.null(Vcov)) {
-            if (is.null(obj)) 
-              mess = "Both  'obj' and 'Vcov' are missing"
-            else {
-              clm = class(obj)
-              part1 = "There are no vcov() methods for model objects of class \""
-              mess = paste0(part1, clm, "\".\nInput the 'Vcov' parameter.")
-            }
-            stop(mess)
-          }
-          if (length(texts) > 1) { kkk = texts[1]
-          } else kkk = strsplit(texts[1], ";")[[1]]
-          kkkfl = as.formula(paste("~", kkk[1]))
-          vvss = setdiff(all.vars(kkkfl), "x")
-          texts = getFromNamespace(".smartsub", ns="nlWaldTest")(vvss, "b", texts)
-          if (length(texts) > 1) { ltext0 = texts
-          } else ltext0 = strsplit(texts, ";")[[1]]
-          texts1 = gsub("[", "", texts, fixed = T)
-          texts1 = gsub("]", "", texts1, fixed = T)
-          if (length(texts1) > 1) { ltext = texts1
-          } else ltext = strsplit(texts1, ";")[[1]]
-          r = length(ltext)
-          n = length(coeff)
-          namess = paste0("b", 1:n)
-          for (j in 1L:n) assign(namess[j], coeff[j])
-          if (!is.null(x)) {
-            nx = length(x)
-            namesx = paste0("x", 1:nx)
-            for (j in 1L:nx) assign(namesx[j], x[j])
-          }
-          grad = c()
-          hess = c()
-          for (i in 1L:r) {
-            if (!is.null(parameters)) {
-              fli <- as.formula(paste("~", paste0(gsub(")", "", ltext[i]), ", ", 
-                                                  parameters, ")")))
-            } else {
-              fli <- as.formula(paste("~", ltext[i]))
-            }
-            z = try(deriv(as.formula(fli), namess), silent = T)
-            if (class(z) == "try-error") {
-              tei = as.character(i)
-              tri2 = ", numerical derivatives were used in delta-method"
-              wate = paste0("Note: For function ", i, tri2)
-              message(wate)
-              if (!is.null(parameters)) {
-                ez = numericDeriv(quote(eval(parse(text = paste0(gsub(")", "", ltext[i]), ", ", 
-                                                                 parameters, ")")))), 
-                                  theta=namess)
-              } else {
-                ez = numericDeriv(quote(eval(parse(text = ltext[i]))), 
-                                  theta=namess)
-              }
-            } else ez = eval(z)
-            hessj = attr(ez, "gradient")
-            grad = rbind(grad, ez[1])
-            hess = rbind(hess, hessj)
-          }
-          Rb = grad
-          
-          
-          ddd = hess %*% Vcov %*% t(hess)
-          # On retire ce test
-          # matr = chol2inv(chol(ddd))
-          ses = sqrt(diag(ddd))
-          
-          
-          
-          trydf = identical(df2, T)
-          if (trydf) {
-            isdf = try(df.residual(obj), silent = T)
-            df2 = isdf
-            if (is.null(df2)) {
-              wn = "Note: Failed to extract df for denomenator; z-intervals applied"
-              message(wn)
-            }
-          }
-          getFromNamespace(".getint", ns="nlWaldTest")(as.numeric(Rb), ltext0, ses, level, df = df2)
-        }
-        
-        
-        
-        for (ti in t) {  
-          
-          if (is.null(model_before)) {
-            mb <- "model_before=NULL"
-          } else {
-            mb <- paste0("model_before=\"", model_before, "\"")
-          }
-          
-          if (is.null(model_after)) {
-            mb <- paste0(mb, ", model_after=NULL")
-          } else {
-            mb <- paste0(mb, ", model_after=\"", model_after, "\"")
-          }
-          
-          if (!is.null(model)) {
-            mb <- paste0(mb, ", model=\"", model, "\"")
-          }
-          
-          if (!is.null(par_add)) {
-            mb <- paste0(mb, ", pfixed=")
-            mb <- paste0(mb, paste0("c(", paste(names(par_add), "=", unname(par_add), collapse = ", "), ")"))
-          }
-          
-          
-          ghi <- suppressMessages ( nlConfint2(texts=paste0(c("try_g2(", paste0("b[", seq_along(par_hess), "], ", collapse=""),"Time=", ti, ")"), 
-                                                            collapse = ""), 
-                                               parameters = mb, 
-                                               level = 0.95, coeff = par_hess,
-                                               Vcov = VCov, df2 = TRUE)
-          )
-          
-          gh <- rbind(gh, data.frame(time=ti, value=ghi[, 1], mean=NA, sd=NA, 
-                                     'X2.5'=ghi[, 2], 'X97.5'=ghi[, 3]))
-          
-        }
-        
-        return(gh)
-      } else {
-        stop("Or nlWaldTest or MultiRNG packages must be present for distribution estimation")
-      }
+      tgm <- Tagloss_model(Time, par=c(par, pfixed), 
+                           model_before=model_before,
+                           model_after=model_after, 
+                           Hessian = NULL, model = model)
+      
+      return(tgm)
     }
+    
+    
+    nlConfint2 <- function (obj = NULL, texts, level = 0.95, coeff = NULL, Vcov = NULL, 
+                            df2 = NULL, x = NULL, parameters=NULL) 
+    {
+      if (!is.null(obj)) {
+        co = try(coef(obj), silent = T)
+        cond = attr(co, "condition")
+        if (is.null(coeff) && (is.null(cond))) 
+          coeff = co
+        vc = try(vcov(obj), silent = T)
+        cond2 = attr(vc, "condition")
+        if (is.null(Vcov) && (is.null(cond2))) 
+          Vcov = vc
+      }
+      if (is.null(coeff)) {
+        if (is.null(obj)) 
+          mess = "Both  'obj' and 'coeff' are missing"
+        else {
+          clm <- class(obj)
+          part1 <- "There are no coef() methods for model objects of class \""
+          mess <- paste0(part1, clm, "\".\nInput the 'coeff' parameter.")
+        }
+        stop(mess)
+      }
+      if (is.null(Vcov)) {
+        if (is.null(obj)) 
+          mess = "Both  'obj' and 'Vcov' are missing"
+        else {
+          clm <- class(obj)
+          part1 <- "There are no vcov() methods for model objects of class \""
+          mess <- paste0(part1, clm, "\".\nInput the 'Vcov' parameter.")
+        }
+        stop(mess)
+      }
+      if (length(texts) > 1) { kkk = texts[1]
+      } else kkk = strsplit(texts[1], ";")[[1]]
+      kkkfl = as.formula(paste("~", kkk[1]))
+      vvss = setdiff(all.vars(kkkfl), "x")
+      texts = getFromNamespace(".smartsub", ns="nlWaldTest")(vvss, "b", texts)
+      if (length(texts) > 1) { ltext0 = texts
+      } else ltext0 = strsplit(texts, ";")[[1]]
+      texts1 = gsub("[", "", texts, fixed = T)
+      texts1 = gsub("]", "", texts1, fixed = T)
+      if (length(texts1) > 1) { ltext = texts1
+      } else ltext = strsplit(texts1, ";")[[1]]
+      r = length(ltext)
+      n = length(coeff)
+      namess = paste0("b", 1:n)
+      for (j in 1L:n) assign(namess[j], coeff[j])
+      if (!is.null(x)) {
+        nx = length(x)
+        namesx = paste0("x", 1:nx)
+        for (j in 1L:nx) assign(namesx[j], x[j])
+      }
+      grad = c()
+      hess = c()
+      for (i in 1L:r) {
+        if (!is.null(parameters)) {
+          fli <- as.formula(paste("~", paste0(gsub(")", "", ltext[i]), ", ", 
+                                              parameters, ")")))
+        } else {
+          fli <- as.formula(paste("~", ltext[i]))
+        }
+        z = try(deriv(as.formula(fli), namess), silent = T)
+        if (inherits(z, "try-error")) {
+          tei = as.character(i)
+          tri2 = ", numerical derivatives were used in delta-method"
+          wate = paste0("Note: For function ", i, tri2)
+          message(wate)
+          if (!is.null(parameters)) {
+            ez = numericDeriv(quote(eval(parse(text = paste0(gsub(")", "", ltext[i]), ", ", 
+                                                             parameters, ")")))), 
+                              theta=namess)
+          } else {
+            ez = numericDeriv(quote(eval(parse(text = ltext[i]))), 
+                              theta=namess)
+          }
+        } else ez = eval(z)
+        hessj = attr(ez, "gradient")
+        grad = rbind(grad, ez[1])
+        hess = rbind(hess, hessj)
+      }
+      Rb = grad
+      
+      
+      ddd = hess %*% Vcov %*% t(hess)
+      # On retire ce test
+      # matr = chol2inv(chol(ddd))
+      ses = sqrt(diag(ddd))
+      
+      
+      
+      trydf = identical(df2, T)
+      if (trydf) {
+        isdf = try(df.residual(obj), silent = T)
+        df2 = isdf
+        if (is.null(df2)) {
+          wn = "Note: Failed to extract df for denomenator; z-intervals applied"
+          message(wn)
+        }
+      }
+      getFromNamespace(".getint", ns="nlWaldTest")(as.numeric(Rb), ltext0, ses, level, df = df2)
+    }
+    
+    for (ti in t) {  
+      
+      if (is.null(model_before)) {
+        mb <- "model_before=NULL"
+      } else {
+        mb <- paste0("model_before=\"", model_before, "\"")
+      }
+      
+      if (is.null(model_after)) {
+        mb <- paste0(mb, ", model_after=NULL")
+      } else {
+        mb <- paste0(mb, ", model_after=\"", model_after, "\"")
+      }
+      
+      if (!is.null(model)) {
+        mb <- paste0(mb, ", model=\"", model, "\"")
+      }
+      
+      if (!is.null(par_add)) {
+        mb <- paste0(mb, ", pfixed=")
+        mb <- paste0(mb, paste0("c(", paste(names(par_add), "=", unname(par_add), collapse = ", "), ")"))
+      }
+      
+      
+      ghi <- suppressMessages ( nlConfint2(texts=paste0(c("try_g2(", paste0("b[", seq_along(par_hess), "], ", collapse=""),"Time=", ti, ")"), 
+                                                        collapse = ""), 
+                                           parameters = mb, 
+                                           level = 0.95, coeff = par_hess,
+                                           Vcov = VCov, df2 = TRUE)
+      )
+      
+      gh <- rbind(gh, data.frame(time=ti, value=ghi[, 1], mean=NA, sd=NA, 
+                                 'X2.5'=ghi[, 2], 'X97.5'=ghi[, 3]))
+      
+    }
+    
+    return(gh)
   }
+  
+  # Ce n'est pas null et pas delta; je peux utiliser RandomFromHessianOrMCMC
+  
+  dfr <- RandomFromHessianOrMCMC(method = method, mcmc=mcmc, Hessian = Hessian, 
+                                 fitted.parameters = par, replicates = replicates, 
+                                 fn=Tagloss_model, ParTofn = "par",  model_before = model_before, 
+                                 model_after = model_after, 
+                                 model=model, x=x, t=t)
+  return(dfr$quantiles)
 }
